@@ -3,7 +3,6 @@ import re
 import subprocess
 from enum import Enum
 from pathlib import Path
-from typing import Optional, Union
 
 import click
 import shellingham
@@ -200,9 +199,9 @@ def install_powershell(*, prog_name: str, complete_var: str, shell: str) -> Path
 
 
 def install(
-    shell: Optional[str] = None,
-    prog_name: Optional[str] = None,
-    complete_var: Optional[str] = None,
+    shell: str | None = None,
+    prog_name: str | None = None,
+    complete_var: str | None = None,
 ) -> tuple[str, Path]:
     prog_name = prog_name or click.get_current_context().find_root().info_name
     assert prog_name
@@ -236,13 +235,13 @@ def install(
         raise click.exceptions.Exit(1)
 
 
-def _get_shell_name() -> Union[str, None]:
+def _get_shell_name() -> str | None:
     """Get the current shell name, if available.
 
     The name will always be lowercase. If the shell cannot be detected, None is
     returned.
     """
-    name: Union[str, None]  # N.B. shellingham is untyped
+    name: str | None  # N.B. shellingham is untyped
     try:
         # N.B. detect_shell returns a tuple of (shell name, shell command).
         # We only need the name.
