@@ -5,14 +5,14 @@ import shutil
 import subprocess
 import sys
 import traceback
-from collections.abc import Sequence
+from collections.abc import Callable, Sequence
 from datetime import datetime
 from enum import Enum
 from functools import update_wrapper
 from pathlib import Path
 from traceback import FrameSummary, StackSummary
 from types import TracebackType
-from typing import Annotated, Any, Callable, Optional, Union
+from typing import Annotated, Any
 from uuid import UUID
 
 import click
@@ -57,9 +57,9 @@ _typer_developer_exception_attr_name = "__typer_developer_exception__"
 
 
 def except_hook(
-    exc_type: type[BaseException], exc_value: BaseException, tb: Optional[TracebackType]
+    exc_type: type[BaseException], exc_value: BaseException, tb: TracebackType | None
 ) -> None:
-    exception_config: Union[DeveloperExceptionConfig, None] = getattr(
+    exception_config: DeveloperExceptionConfig | None = getattr(
         exc_value, _typer_developer_exception_attr_name, None
     )
     standard_traceback = os.getenv(
@@ -73,7 +73,7 @@ def except_hook(
         _original_except_hook(exc_type, exc_value, tb)
         return
     typer_path = os.path.dirname(__file__)
-    click_path = os.path.dirname(click.__file__)  # ty: ignore[no-matching-overload]
+    click_path = os.path.dirname(click.__file__)
     internal_dir_names = [typer_path, click_path]
     exc = exc_value
     if HAS_RICH:
@@ -134,7 +134,7 @@ class Typer:
         self,
         *,
         name: Annotated[
-            Optional[str],
+            str | None,
             Doc(
                 """
                 The name of this application.
@@ -151,7 +151,7 @@ class Typer:
             ),
         ] = Default(None),
         cls: Annotated[
-            Optional[type[TyperGroup]],
+            type[TyperGroup] | None,
             Doc(
                 """
                 The class of this app. Mainly used when [using the Click library underneath](https://typer.tiangolo.com/tutorial/using-click/). Can usually be left at the default value `None`.
@@ -200,7 +200,7 @@ class Typer:
             ),
         ] = Default(False),
         subcommand_metavar: Annotated[
-            Optional[str],
+            str | None,
             Doc(
                 """
                 **Note**: you probably shouldn't use this parameter, it is inherited
@@ -226,7 +226,7 @@ class Typer:
             ),
         ] = Default(False),
         result_callback: Annotated[
-            Optional[Callable[..., Any]],
+            Callable[..., Any] | None,
             Doc(
                 """
                 **Note**: you probably shouldn't use this parameter, it is inherited
@@ -240,7 +240,7 @@ class Typer:
         ] = Default(None),
         # Command
         context_settings: Annotated[
-            Optional[dict[Any, Any]],
+            dict[Any, Any] | None,
             Doc(
                 """
                 Pass configurations for the [context](https://typer.tiangolo.com/tutorial/commands/context/).
@@ -257,7 +257,7 @@ class Typer:
             ),
         ] = Default(None),
         callback: Annotated[
-            Optional[Callable[..., Any]],
+            Callable[..., Any] | None,
             Doc(
                 """
                 Add a callback to the main Typer app. Can be overridden with `@app.callback()`.
@@ -277,7 +277,7 @@ class Typer:
             ),
         ] = Default(None),
         help: Annotated[
-            Optional[str],
+            str | None,
             Doc(
                 """
                 Help text for the main Typer app.
@@ -295,7 +295,7 @@ class Typer:
             ),
         ] = Default(None),
         epilog: Annotated[
-            Optional[str],
+            str | None,
             Doc(
                 """
                 Text that will be printed right after the help text.
@@ -311,7 +311,7 @@ class Typer:
             ),
         ] = Default(None),
         short_help: Annotated[
-            Optional[str],
+            str | None,
             Doc(
                 """
                 A shortened version of the help text that can be used e.g. in the help table listing subcommands.
@@ -426,7 +426,7 @@ class Typer:
             ),
         ] = DEFAULT_MARKUP_MODE,
         rich_help_panel: Annotated[
-            Union[str, None],
+            str | None,
             Doc(
                 """
                 Set the panel name of the command when the help is printed with Rich.
@@ -542,13 +542,13 @@ class Typer:
         )
         self.registered_groups: list[TyperInfo] = []
         self.registered_commands: list[CommandInfo] = []
-        self.registered_callback: Optional[TyperInfo] = None
+        self.registered_callback: TyperInfo | None = None
 
     def callback(
         self,
         *,
         cls: Annotated[
-            Optional[type[TyperGroup]],
+            type[TyperGroup] | None,
             Doc(
                 """
                 The class of this app. Mainly used when [using the Click library underneath](https://typer.tiangolo.com/tutorial/using-click/). Can usually be left at the default value `None`.
@@ -573,7 +573,7 @@ class Typer:
             ),
         ] = Default(False),
         subcommand_metavar: Annotated[
-            Optional[str],
+            str | None,
             Doc(
                 """
                 **Note**: you probably shouldn't use this parameter, it is inherited
@@ -599,7 +599,7 @@ class Typer:
             ),
         ] = Default(False),
         result_callback: Annotated[
-            Optional[Callable[..., Any]],
+            Callable[..., Any] | None,
             Doc(
                 """
                 **Note**: you probably shouldn't use this parameter, it is inherited
@@ -613,7 +613,7 @@ class Typer:
         ] = Default(None),
         # Command
         context_settings: Annotated[
-            Optional[dict[Any, Any]],
+            dict[Any, Any] | None,
             Doc(
                 """
                 Pass configurations for the [context](https://typer.tiangolo.com/tutorial/commands/context/).
@@ -622,7 +622,7 @@ class Typer:
             ),
         ] = Default(None),
         help: Annotated[
-            Optional[str],
+            str | None,
             Doc(
                 """
                 Help text for the command.
@@ -632,7 +632,7 @@ class Typer:
             ),
         ] = Default(None),
         epilog: Annotated[
-            Optional[str],
+            str | None,
             Doc(
                 """
                 Text that will be printed right after the help text.
@@ -640,7 +640,7 @@ class Typer:
             ),
         ] = Default(None),
         short_help: Annotated[
-            Optional[str],
+            str | None,
             Doc(
                 """
                 A shortened version of the help text that can be used e.g. in the help table listing subcommands.
@@ -649,7 +649,7 @@ class Typer:
             ),
         ] = Default(None),
         options_metavar: Annotated[
-            Optional[str],
+            str | None,
             Doc(
                 """
                 In the example usage string of the help text for a command, the default placeholder for various arguments is `[OPTIONS]`.
@@ -688,7 +688,7 @@ class Typer:
         ] = Default(False),
         # Rich settings
         rich_help_panel: Annotated[
-            Union[str, None],
+            str | None,
             Doc(
                 """
                 Set the panel name of the command when the help is printed with Rich.
@@ -751,7 +751,7 @@ class Typer:
     def command(
         self,
         name: Annotated[
-            Optional[str],
+            str | None,
             Doc(
                 """
                 The name of this command.
@@ -760,7 +760,7 @@ class Typer:
         ] = None,
         *,
         cls: Annotated[
-            Optional[type[TyperCommand]],
+            type[TyperCommand] | None,
             Doc(
                 """
                 The class of this command. Mainly used when [using the Click library underneath](https://typer.tiangolo.com/tutorial/using-click/). Can usually be left at the default value `None`.
@@ -769,7 +769,7 @@ class Typer:
             ),
         ] = None,
         context_settings: Annotated[
-            Optional[dict[Any, Any]],
+            dict[Any, Any] | None,
             Doc(
                 """
                 Pass configurations for the [context](https://typer.tiangolo.com/tutorial/commands/context/).
@@ -778,7 +778,7 @@ class Typer:
             ),
         ] = None,
         help: Annotated[
-            Optional[str],
+            str | None,
             Doc(
                 """
                 Help text for the command.
@@ -788,7 +788,7 @@ class Typer:
             ),
         ] = None,
         epilog: Annotated[
-            Optional[str],
+            str | None,
             Doc(
                 """
                 Text that will be printed right after the help text.
@@ -796,7 +796,7 @@ class Typer:
             ),
         ] = None,
         short_help: Annotated[
-            Optional[str],
+            str | None,
             Doc(
                 """
                 A shortened version of the help text that can be used e.g. in the help table listing subcommands.
@@ -805,7 +805,7 @@ class Typer:
             ),
         ] = None,
         options_metavar: Annotated[
-            Optional[str],
+            str | None,
             Doc(
                 """
                 In the example usage string of the help text for a command, the default placeholder for various arguments is `[OPTIONS]`.
@@ -852,7 +852,7 @@ class Typer:
         ] = False,
         # Rich settings
         rich_help_panel: Annotated[
-            Union[str, None],
+            str | None,
             Doc(
                 """
                 Set the panel name of the command when the help is printed with Rich.
@@ -915,7 +915,7 @@ class Typer:
         typer_instance: "Typer",
         *,
         name: Annotated[
-            Optional[str],
+            str | None,
             Doc(
                 """
                 The name of this subcommand.
@@ -925,7 +925,7 @@ class Typer:
             ),
         ] = Default(None),
         cls: Annotated[
-            Optional[type[TyperGroup]],
+            type[TyperGroup] | None,
             Doc(
                 """
                 The class of this subcommand. Mainly used when [using the Click library underneath](https://typer.tiangolo.com/tutorial/using-click/). Can usually be left at the default value `None`.
@@ -950,7 +950,7 @@ class Typer:
             ),
         ] = Default(False),
         subcommand_metavar: Annotated[
-            Optional[str],
+            str | None,
             Doc(
                 """
                 **Note**: you probably shouldn't use this parameter, it is inherited
@@ -976,7 +976,7 @@ class Typer:
             ),
         ] = Default(False),
         result_callback: Annotated[
-            Optional[Callable[..., Any]],
+            Callable[..., Any] | None,
             Doc(
                 """
                 **Note**: you probably shouldn't use this parameter, it is inherited
@@ -990,7 +990,7 @@ class Typer:
         ] = Default(None),
         # Command
         context_settings: Annotated[
-            Optional[dict[Any, Any]],
+            dict[Any, Any] | None,
             Doc(
                 """
                 Pass configurations for the [context](https://typer.tiangolo.com/tutorial/commands/context/).
@@ -999,7 +999,7 @@ class Typer:
             ),
         ] = Default(None),
         callback: Annotated[
-            Optional[Callable[..., Any]],
+            Callable[..., Any] | None,
             Doc(
                 """
                 Add a callback to this app.
@@ -1008,7 +1008,7 @@ class Typer:
             ),
         ] = Default(None),
         help: Annotated[
-            Optional[str],
+            str | None,
             Doc(
                 """
                 Help text for the subcommand.
@@ -1018,7 +1018,7 @@ class Typer:
             ),
         ] = Default(None),
         epilog: Annotated[
-            Optional[str],
+            str | None,
             Doc(
                 """
                 Text that will be printed right after the help text.
@@ -1026,7 +1026,7 @@ class Typer:
             ),
         ] = Default(None),
         short_help: Annotated[
-            Optional[str],
+            str | None,
             Doc(
                 """
                 A shortened version of the help text that can be used e.g. in the help table listing subcommands.
@@ -1035,7 +1035,7 @@ class Typer:
             ),
         ] = Default(None),
         options_metavar: Annotated[
-            Optional[str],
+            str | None,
             Doc(
                 """
                 In the example usage string of the help text for a command, the default placeholder for various arguments is `[OPTIONS]`.
@@ -1074,7 +1074,7 @@ class Typer:
         ] = False,
         # Rich settings
         rich_help_panel: Annotated[
-            Union[str, None],
+            str | None,
             Doc(
                 """
                 Set the panel name of the command when the help is printed with Rich.
@@ -1361,8 +1361,8 @@ def get_command_name(name: str) -> str:
 
 
 def get_params_convertors_ctx_param_name_from_function(
-    callback: Optional[Callable[..., Any]],
-) -> tuple[list[Union[click.Argument, click.Option]], dict[str, Any], Optional[str]]:
+    callback: Callable[..., Any] | None,
+) -> tuple[list[click.Argument | click.Option], dict[str, Any], str | None]:
     params = []
     convertors = {}
     context_param_name = None
@@ -1386,7 +1386,7 @@ def get_command_from_info(
     rich_markup_mode: MarkupMode,
 ) -> click.Command:
     assert command_info.callback, "A command must have a callback function"
-    name = command_info.name or get_command_name(command_info.callback.__name__)  # ty: ignore[possibly-missing-attribute]
+    name = command_info.name or get_command_name(command_info.callback.__name__)  # ty:ignore[unresolved-attribute]
     use_help = command_info.help
     if use_help is None:
         use_help = inspect.getdoc(command_info.callback)
@@ -1424,8 +1424,8 @@ def get_command_from_info(
     return command
 
 
-def determine_type_convertor(type_: Any) -> Optional[Callable[[Any], Any]]:
-    convertor: Optional[Callable[[Any], Any]] = None
+def determine_type_convertor(type_: Any) -> Callable[[Any], Any] | None:
+    convertor: Callable[[Any], Any] | None = None
     if lenient_issubclass(type_, Path):
         convertor = param_path_convertor
     if lenient_issubclass(type_, Enum):
@@ -1433,7 +1433,7 @@ def determine_type_convertor(type_: Any) -> Optional[Callable[[Any], Any]]:
     return convertor
 
 
-def param_path_convertor(value: Optional[str] = None) -> Optional[Path]:
+def param_path_convertor(value: str | None = None) -> Path | None:
     if value is not None:
         # allow returning any subclass of Path created by an annotated parser without converting
         # it back to a Path
@@ -1455,9 +1455,9 @@ def generate_enum_convertor(enum: type[Enum]) -> Callable[[Any], Any]:
 
 
 def generate_list_convertor(
-    convertor: Optional[Callable[[Any], Any]], default_value: Optional[Any]
-) -> Callable[[Optional[Sequence[Any]]], Optional[list[Any]]]:
-    def internal_convertor(value: Optional[Sequence[Any]]) -> Optional[list[Any]]:
+    convertor: Callable[[Any], Any] | None, default_value: Any | None
+) -> Callable[[Sequence[Any] | None], list[Any] | None]:
+    def internal_convertor(value: Sequence[Any] | None) -> list[Any] | None:
         if (value is None) or (default_value is None and len(value) == 0):
             return None
         return [convertor(v) if convertor else v for v in value]
@@ -1467,17 +1467,17 @@ def generate_list_convertor(
 
 def generate_tuple_convertor(
     types: Sequence[Any],
-) -> Callable[[Optional[tuple[Any, ...]]], Optional[tuple[Any, ...]]]:
+) -> Callable[[tuple[Any, ...] | None], tuple[Any, ...] | None]:
     convertors = [determine_type_convertor(type_) for type_ in types]
 
     def internal_convertor(
-        param_args: Optional[tuple[Any, ...]],
-    ) -> Optional[tuple[Any, ...]]:
+        param_args: tuple[Any, ...] | None,
+    ) -> tuple[Any, ...] | None:
         if param_args is None:
             return None
         return tuple(
             convertor(arg) if convertor else arg
-            for (convertor, arg) in zip(convertors, param_args)
+            for (convertor, arg) in zip(convertors, param_args, strict=False)
         )
 
     return internal_convertor
@@ -1485,12 +1485,12 @@ def generate_tuple_convertor(
 
 def get_callback(
     *,
-    callback: Optional[Callable[..., Any]] = None,
+    callback: Callable[..., Any] | None = None,
     params: Sequence[click.Parameter] = [],
-    convertors: Optional[dict[str, Callable[[str], Any]]] = None,
-    context_param_name: Optional[str] = None,
+    convertors: dict[str, Callable[[str], Any]] | None = None,
+    context_param_name: str | None = None,
     pretty_exceptions_short: bool,
-) -> Optional[Callable[..., Any]]:
+) -> Callable[..., Any] | None:
     use_convertors = convertors or {}
     if not callback:
         return None
@@ -1620,15 +1620,13 @@ def get_click_type(
     raise RuntimeError(f"Type not yet supported: {annotation}")  # pragma: no cover
 
 
-def lenient_issubclass(
-    cls: Any, class_or_tuple: Union[AnyType, tuple[AnyType, ...]]
-) -> bool:
+def lenient_issubclass(cls: Any, class_or_tuple: AnyType | tuple[AnyType, ...]) -> bool:
     return isinstance(cls, type) and issubclass(cls, class_or_tuple)
 
 
 def get_click_param(
     param: ParamMeta,
-) -> tuple[Union[click.Argument, click.Option], Any]:
+) -> tuple[click.Argument | click.Option, Any]:
     # First, find out what will be:
     # * ParamInfo (ArgumentInfo or OptionInfo)
     # * default_value
@@ -1791,9 +1789,9 @@ def get_click_param(
 
 def get_param_callback(
     *,
-    callback: Optional[Callable[..., Any]] = None,
-    convertor: Optional[Callable[..., Any]] = None,
-) -> Optional[Callable[..., Any]]:
+    callback: Callable[..., Any] | None = None,
+    convertor: Callable[..., Any] | None = None,
+) -> Callable[..., Any] | None:
     if not callback:
         return None
     parameters = get_params_from_function(callback)
@@ -1842,8 +1840,8 @@ def get_param_callback(
 
 
 def get_param_completion(
-    callback: Optional[Callable[..., Any]] = None,
-) -> Optional[Callable[..., Any]]:
+    callback: Callable[..., Any] | None = None,
+) -> Callable[..., Any] | None:
     if not callback:
         return None
     parameters = get_params_from_function(callback)
@@ -1880,7 +1878,7 @@ def get_param_completion(
             f"Invalid autocompletion callback parameters: {show_params}"
         )
 
-    def wrapper(ctx: click.Context, args: list[str], incomplete: Optional[str]) -> Any:
+    def wrapper(ctx: click.Context, args: list[str], incomplete: str | None) -> Any:
         use_params: dict[str, Any] = {}
         if ctx_name:
             use_params[ctx_name] = ctx
