@@ -1,7 +1,7 @@
 import inspect
-import sys
+from collections.abc import Callable
 from copy import copy
-from typing import Any, Callable, Optional, cast
+from typing import Any, cast
 
 from ._typing import Annotated, get_args, get_origin, get_type_hints
 from .models import ArgumentInfo, OptionInfo, ParameterInfo, ParamMeta
@@ -105,11 +105,7 @@ def _split_annotation_from_typer_annotations(
 
 
 def get_params_from_function(func: Callable[..., Any]) -> dict[str, ParamMeta]:
-    if sys.version_info >= (3, 10):
-        signature = inspect.signature(func, eval_str=True)
-    else:
-        signature = inspect.signature(func)
-
+    signature = inspect.signature(func, eval_str=True)
     type_hints = get_type_hints(func)
     params = {}
     for param in signature.parameters.values():
@@ -190,7 +186,7 @@ def get_params_from_function(func: Callable[..., Any]) -> dict[str, ParamMeta]:
     return params
 
 
-def parse_boolean_env_var(env_var_value: Optional[str], default: bool) -> bool:
+def parse_boolean_env_var(env_var_value: str | None, default: bool) -> bool:
     if env_var_value is None:
         return default
     value = env_var_value.lower()
